import {Layer, Item, Value} from 'dripcap';
import Enum from 'driptool/enum';

export default class UTPDissector {
  static get namespaces() {
    return [
      '::Ethernet::IPv4::UDP',
      '::Ethernet::IPv6::UDP'
    ];
  }

  analyze(packet, parentLayer) {
    let layer = {
      items: [],
      confidence: 0.6,
      namespace: parentLayer.namespace + '::uTP',
      name: 'μTP',
      id: 'utp'
    };

    let firstByte = parentLayer.payload.readUInt8(0);
    let version = firstByte & 0b00001111;

    if (version !== 1)
      throw new Error('wrong uTP version');

    let table = {
      0: 'ST_DATA',
      1: 'ST_FIN',
      2: 'ST_STATE',
      3: 'ST_RESET',
      4: 'ST_SYN'
    };

    let typeNumber = firstByte >> 4;
    let type = new Enum(table, typeNumber);

    if (!(typeNumber in table))
      throw new Error('wrong uTP type')

    layer.items.push({
      name: 'Version',
      id: 'version',
      range: '0:1',
      value: version
    });

    layer.items.push({
      name: 'Type',
      id: 'type',
      range: '0:1',
      value: typeNumber,
      summary: type.toString()
    });

    let extTable = {
      0: 'none',
      1: 'Selective acks'
    }

    let firstExtensionNumber = parentLayer.payload.readUInt8(1);
    let firstExtension = new Enum(extTable, firstExtensionNumber);
    layer.items.push({
      name: 'First Extension',
      id: 'firstExtension',
      range: '1:2',
      value: firstExtensionNumber,
      summary: firstExtension.toString()
    });

    let connectionID = parentLayer.payload.readUInt16BE(2);
    layer.items.push({
      name: 'Connection ID',
      id: 'id',
      range: '2:4',
      value: connectionID
    });

    let timestamp = parentLayer.payload.readUInt32BE(4);
    layer.items.push({
      name: 'Timestamp Microseconds',
      id: 'timestamp',
      range: '4:8',
      value: timestamp
    });

    let timestampDiff = parentLayer.payload.readUInt32BE(8);
    layer.items.push({
      name: 'Timestamp Difference Microseconds',
      id: 'timestampDiff',
      range: '8:12',
      value: timestampDiff
    });

    let windowSize = parentLayer.payload.readUInt32BE(12);
    layer.items.push({
      name: 'Window Size',
      id: 'windowSize',
      range: '12:16',
      value: windowSize
    });

    let seq = parentLayer.payload.readUInt16BE(16);
    layer.items.push({
      name: 'Sequence Number',
      id: 'seq',
      range: '16:18',
      value: seq
    });

    let ack = parentLayer.payload.readUInt16BE(18);
    layer.items.push({
      name: 'Acknowledgment Number',
      id: 'ack',
      range: '18:20',
      value: ack
    });

    let offset = 20;
    layer.range = offset + ':';
    layer.payload = parentLayer.payload.slice(offset);

    layer.items.push({
      name: 'Payload',
      id: 'payload',
      range: offset + ':',
      value: layer.payload
    });

    layer.summary = `[${type.toString()}] seq:${seq} ack:${ack}`;
    return new Layer(layer);
  }
}
